@@ -1,7 +1,8 @@
-#encoding: utf-8
+# frozen_string_literal: true
+
 #
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2019  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -280,7 +281,7 @@ EXPECTED
 This is a table with trailing whitespace in one row:
 
 |cell11|cell12|
-|cell21|cell22| 
+|cell21|cell22|
 |cell31|cell32|
 RAW
 
@@ -382,67 +383,76 @@ EXPECTED
     expected = '<p><img src="/images/comment.png&quot;onclick=&amp;#x61;&amp;#x6c;&amp;#x65;&amp;#x72;&amp;#x74;&amp;#x28;&amp;#x27;&amp;#x58;&amp;#x53;&amp;#x53;&amp;#x27;&amp;#x29;;&amp;#x22;" alt="" /></p>'
     assert_equal expected.gsub(%r{\s+}, ''), to_html(raw).gsub(%r{\s+}, '')
   end
-  
-  
+
   STR_WITHOUT_PRE = [
-  # 0
-"h1. Title
+    # 0
+    <<~STR.chomp,
+      h1. Title
 
-Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Maecenas sed libero.",
-  # 1
-"h2. Heading 2
+      Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Maecenas sed libero.
+    STR
+    # 1
+    <<~STR.chomp,
+      h2. Heading 2
 
-Maecenas sed elit sit amet mi accumsan vestibulum non nec velit. Proin porta tincidunt lorem, consequat rhoncus dolor fermentum in.
+      Maecenas sed elit sit amet mi accumsan vestibulum non nec velit. Proin porta tincidunt lorem, consequat rhoncus dolor fermentum in.
 
-Cras ipsum felis, ultrices at porttitor vel, faucibus eu nunc.",
-  # 2
-"h2. Heading 2
+      Cras ipsum felis, ultrices at porttitor vel, faucibus eu nunc.
+    STR
+    # 2
+    <<~STR.chomp,
+      h2. Heading 2
 
-Morbi facilisis accumsan orci non pharetra.
+      Morbi facilisis accumsan orci non pharetra.
 
-h3. Heading 3
+      h3. Heading 3
 
-Nulla nunc nisi, egestas in ornare vel, posuere ac libero.",
-  # 3
-"h3. Heading 3
+      Nulla nunc nisi, egestas in ornare vel, posuere ac libero.
+    STR
+    # 3
+    <<~STR.chomp,
+      h3. Heading 3
 
-Praesent eget turpis nibh, a lacinia nulla.",
-  # 4
-"h2. Heading 2
-
-Ut rhoncus elementum adipiscing."]
+      Praesent eget turpis nibh, a lacinia nulla.
+    STR
+    # 4
+    <<~STR.chomp,
+      h2. Heading 2
 
+      Ut rhoncus elementum adipiscing.
+    STR
+  ]
   TEXT_WITHOUT_PRE = STR_WITHOUT_PRE.join("\n\n").freeze
-  
+
   def test_get_section_should_return_the_requested_section_and_its_hash
     assert_section_with_hash STR_WITHOUT_PRE[1], TEXT_WITHOUT_PRE, 2
     assert_section_with_hash STR_WITHOUT_PRE[2..3].join("\n\n"), TEXT_WITHOUT_PRE, 3
     assert_section_with_hash STR_WITHOUT_PRE[3], TEXT_WITHOUT_PRE, 5
     assert_section_with_hash STR_WITHOUT_PRE[4], TEXT_WITHOUT_PRE, 6
-    
+
     assert_section_with_hash '', TEXT_WITHOUT_PRE, 0
     assert_section_with_hash '', TEXT_WITHOUT_PRE, 10
   end
-  
+
   def test_update_section_should_update_the_requested_section
     replacement = "New text"
-    
+
     assert_equal [STR_WITHOUT_PRE[0], replacement, STR_WITHOUT_PRE[2..4]].flatten.join("\n\n"), @formatter.new(TEXT_WITHOUT_PRE).update_section(2, replacement)
     assert_equal [STR_WITHOUT_PRE[0..1], replacement, STR_WITHOUT_PRE[4]].flatten.join("\n\n"), @formatter.new(TEXT_WITHOUT_PRE).update_section(3, replacement)
     assert_equal [STR_WITHOUT_PRE[0..2], replacement, STR_WITHOUT_PRE[4]].flatten.join("\n\n"), @formatter.new(TEXT_WITHOUT_PRE).update_section(5, replacement)
     assert_equal [STR_WITHOUT_PRE[0..3], replacement].flatten.join("\n\n"), @formatter.new(TEXT_WITHOUT_PRE).update_section(6, replacement)
-    
+
     assert_equal TEXT_WITHOUT_PRE, @formatter.new(TEXT_WITHOUT_PRE).update_section(0, replacement)
     assert_equal TEXT_WITHOUT_PRE, @formatter.new(TEXT_WITHOUT_PRE).update_section(10, replacement)
   end
-  
+
   def test_update_section_with_hash_should_update_the_requested_section
     replacement = "New text"
-    
+
     assert_equal [STR_WITHOUT_PRE[0], replacement, STR_WITHOUT_PRE[2..4]].flatten.join("\n\n"),
       @formatter.new(TEXT_WITHOUT_PRE).update_section(2, replacement, Digest::MD5.hexdigest(STR_WITHOUT_PRE[1]))
   end
-  
+
   def test_update_section_with_wrong_hash_should_raise_an_error
     assert_raise Redmine::WikiFormatting::StaleSectionError do
       @formatter.new(TEXT_WITHOUT_PRE).update_section(2, "New text", Digest::MD5.hexdigest("Old text"))
@@ -450,38 +460,45 @@ Ut rhoncus elementum adipiscing."]
   end
 
   STR_WITH_PRE = [
-  # 0
-"h1. Title
+    # 0
+    <<~STR.chomp,
+      h1. Title
 
-Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Maecenas sed libero.",
-  # 1
-"h2. Heading 2
+      Lorem ipsum dolor sit amet, consectetuer adipiscing elit. Maecenas sed libero.
+    STR
+    # 1
+    <<~STR.chomp,
+      h2. Heading 2
 
-<pre><code class=\"ruby\">
-  def foo
-  end
-</code></pre>
+      <pre><code class=\"ruby\">
+        def foo
+        end
+      </code></pre>
 
-<pre><code><pre><code class=\"ruby\">
-  Place your code here.
-</code></pre>
-</code></pre>
+      <pre><code><pre><code class=\"ruby\">
+        Place your code here.
+      </code></pre>
+      </code></pre>
 
-Morbi facilisis accumsan orci non pharetra.
+      Morbi facilisis accumsan orci non pharetra.
 
-<pre>
-Pre Content:
+      <pre>
+      Pre Content:
 
-h2. Inside pre
+      h2. Inside pre
 
-<tag> inside pre block
+      <tag> inside pre block
 
-Morbi facilisis accumsan orci non pharetra.
-</pre>",
-  # 2
-"h3. Heading 3
+      Morbi facilisis accumsan orci non pharetra.
+      </pre>
+    STR
+    # 2
+    <<~STR.chomp,
+      h3. Heading 3
 
-Nulla nunc nisi, egestas in ornare vel, posuere ac libero."]
+      Nulla nunc nisi, egestas in ornare vel, posuere ac libero.
+    STR
+  ]
 
   def test_get_section_should_ignore_pre_content
     text = STR_WITH_PRE.join("\n\n")
@@ -493,7 +510,7 @@ Nulla nunc nisi, egestas in ornare vel, posuere ac libero."]
   def test_update_section_should_not_escape_pre_content_outside_section
     text = STR_WITH_PRE.join("\n\n")
     replacement = "New text"
-    
+
     assert_equal [STR_WITH_PRE[0..1], "New text"].flatten.join("\n\n"),
       @formatter.new(text).update_section(3, replacement)
   end
@@ -508,13 +525,13 @@ Content 1
 h1. Heading 2
 
 Content 2
- 
+
 h1. Heading 3
 
 Content 3
 
 h1. Heading 4
- 
+
 Content 4
 STR
 
@@ -540,9 +557,17 @@ STR
   def test_should_not_allow_arbitrary_class_attribute_on_offtags
     %w(code pre kbd).each do |tag|
       assert_html_output({"<#{tag} class=\"foo\">test</#{tag}>" => "<#{tag}>test</#{tag}>"}, false)
+      assert_html_output({"<#{tag} class='foo'>test</#{tag}>" => "<#{tag}>test</#{tag}>"}, false)
+      assert_html_output({"<#{tag} class=\"ruby foo\">test</#{tag}>" => "<#{tag}>test</#{tag}>"}, false)
+      assert_html_output({"<#{tag} class='ruby foo'>test</#{tag}>" => "<#{tag}>test</#{tag}>"}, false)
+      assert_html_output({"<#{tag} class=\"ruby \"foo\" bar\">test</#{tag}>" => "<#{tag}>test</#{tag}>"}, false)
     end
 
     assert_html_output({"<notextile class=\"foo\">test</notextile>" => "test"}, false)
+    assert_html_output({"<notextile class='foo'>test</notextile>" => "test"}, false)
+    assert_html_output({"<notextile class=\"ruby foo\">test</notextile>" => "test"}, false)
+    assert_html_output({"<notextile class='ruby foo'>test</notextile>" => "test"}, false)
+    assert_html_output({"<notextile class=\"ruby \"foo\" bar\">test</notextile>" => "test"}, false)
   end
 
   def test_should_allow_valid_language_class_attribute_on_code_tags
@@ -632,10 +657,10 @@ EXPECTED
   def to_html(text)
     @formatter.new(text).to_html
   end
-  
+
   def assert_section_with_hash(expected, text, index)
     result = @formatter.new(text).get_section(index)
-    
+
     assert_kind_of Array, result
     assert_equal 2, result.size
     assert_equal expected, result.first, "section content did not match"
